.g-control-widget-label
  margin-bottom 2px

.g-control-widget-description
  font-size 12px
  color #666
  margin-bottom 4px

.g-control-item
    input.form-control
      height auto

    select
      appearance none
