.g-candela-container

  .g-candela-vis-setup
    max-width 650px

    .g-candela-panel-group
      position static
      width 100%
      margin 20px 0 0
      overflow-y visible

    .g-candela-update-vis-container
      margin 10px 0
      text-align right

      .g-candela-validation-failed-message
        padding 10px 0

  .g-candela-vis
    height 600px
