.g-item-candela-header
  background-color #f0f0f0
  padding 5px 6px
  color #555
  font-size 16px
  font-weight bold
  margin-top 18px

  select
    display inline
    width auto
    margin-left 25px
