.g-panel-group

  .g-panel
    background-color #ffffff
    opacity 0.95
    margin-top 5px
    border-radius 3px
    height auto
    border 1px solid #d6d6d6

  .g-panel-content
    margin 5px
    overflow hidden

  .g-panel-content:empty
    margin 0

  .g-panel-title-container
    font-weight bold
    text-align left
    background-color #f0f0f0
    cursor pointer
    padding 5px

    .g-panel-title
      display inline-block
      max-width 85%
      overflow hidden

    .g-panel-controls
      float right
