.g-panel-group.g-task-setup-container
  position static
  width 100%
  margin 20px 0 0

  .g-panel-title-container
    padding 6px 10px !important

  .g-panel-content
    padding 5px 6px

.g-task-description-container
  margin-top 10px
  padding-top 12px
  border-top 1px solid #e1e1e1

.g-run-container
  margin 10px 0
  text-align right

  .g-validation-failed-message
    padding 10px 0
