$maxPreviewHeight = 400px

.g-widget-item-previews-wrap
  font-size 12x
  float right
  margin-right 4px
  padding 2px 3px

.g-widget-item-previews-header
  background-color #f0f0f0
  color #555
  font-size 16px
  font-weight bold
  margin-top 10px
  margin-bottom 10px
  padding 5px 6px

.g-widget-item-previews-container
  max-height $maxPreviewHeight
  overflow-x auto
  white-space nowrap
  &.g-wrap
    white-space normal

  img
    max-height $maxPreviewHeight
    border-radius 5px
    width auto

  .panel
    display inline-block
    vertical-align top
    width 30% !important

    .panel-body
      padding 5px

  .json
    background-color #f5f5f5
    border-radius 4px
    border 1px solid #ccc
    color #333
    height 300px
    white-space pre-wrap

.g-widget-item-preview
  display inline
  margin-right 10px
